SystemOrganization addCategory: #Proxies!
SystemOrganization addCategory: #'Proxies-Core'!
SystemOrganization addCategory: #'Proxies-Swapping'!
SystemOrganization addCategory: #'Proxies-Examples'!
SystemOrganization addCategory: #'Proxies-Errors'!
SystemOrganization addCategory: #'Proxies-Examples-Tests'!

TestCase subclass: #MethodWrapperProxyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Examples-Tests'!

!MethodWrapperProxyTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/6/2011 20:40'!
installProxyOn: aClass selector: aSelector. 
	^ self proxyClass on: aClass selector: aSelector ! !

!MethodWrapperProxyTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/6/2011 20:11'!
proxyClass
	^ MethodWrapperProxy! !

!MethodWrapperProxyTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/6/2011 20:10'!
setUp

	self proxyClass runningTest: true.
	! !

!MethodWrapperProxyTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/6/2011 21:28'!
testSimple
	| aProxy |
	
	aProxy := self installProxyOn: ClassWith1Var selector: #foo. 
	
	[self assert: aProxy mareaClass equals: self proxyClass.
	self assert: aProxy mareaIsProxyInstalled.
	self assert: ClassWith1Var new foo equals: 'foo'. 
	self assert: ((ClassWith1Var methodDict at: #foo) literalAt: 1 )equals: 'foo'.
	self assert: (aProxy literalAt: 1) equals: 'foo'.
	aProxy mareaUninstallIfNeeded.
	self deny: aProxy mareaIsProxyInstalled.
	self should: [aProxy literalAt: 1] raise: MessageNotUnderstood.
	] ensure: [ aProxy mareaUninstallIfNeeded].

! !

TestCase subclass: #ObjectProxyTracerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Examples-Tests'!

!ObjectProxyTracerTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 3/30/2011 16:12'!
installProxyOn: anObject. 
	^ self proxyClass installOn: anObject withHandler: ObjectProxyTracerHandler.! !

!ObjectProxyTracerTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 3/30/2011 16:12'!
proxyClass
	^ ObjectProxyWithTarget! !

!ObjectProxyTracerTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 3/30/2011 16:12'!
setUp

	self proxyClass runningTest: true.
	! !

!ObjectProxyTracerTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 3/30/2011 16:57'!
testClassSelector
	| anObject proxy |
	
	anObject := DateAndTime new.
	
	proxy := self installProxyOn: anObject. 
	
	[self assert: proxy mareaClass equals: self proxyClass.
	self assert: proxy class equals: self proxyClass.
	self assert: proxy mareaIsProxyInstalled.
	"all methos that start with marea are understood. And #class too"
	self assert: proxy mareaIsProxyInstalled.
	
	self proxyClass runningTest: false.
	self assert: proxy class equals: DateAndTime.
	self assert: proxy mareaClass equals: self proxyClass.
	
	proxy mareaUninstallIfNeeded.
	self assert: proxy class equals: self proxyClass.
	self assert: proxy mareaClass equals: self proxyClass.
	
	] ensure: [ proxy mareaUninstallIfNeeded].

! !

!ObjectProxyTracerTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 3/30/2011 17:19'!
testIdentity
	| anObject proxy |
	
	anObject := DateAndTime new.
	
	proxy := self installProxyOn: anObject. 
	
	[self assert: proxy mareaClass equals: self proxyClass..
	self assert: proxy mareaIsProxyInstalled.
	self deny: proxy == anObject.
	proxy mareaUninstallIfNeeded.
	self deny: proxy == anObject.

	] ensure: [ anObject mareaIsProxy ifTrue:  [ anObject mareaUninstallIfNeeded]].

! !

!ObjectProxyTracerTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 3/30/2011 19:01'!
testOneSingleObject
	| anObject aProxy |
	
	anObject := 'jejejejej'.
	
	aProxy := self installProxyOn: anObject. 
	
	[self assert: aProxy mareaClass equals: self proxyClass.
	self assert: aProxy mareaIsProxyInstalled.
	self assert: (aProxy includesSubString: 'jeje').
	self assert: aProxy= anObject.
	aProxy mareaUninstallIfNeeded.
	self deny: aProxy mareaIsProxyInstalled.
	self deny: aProxy = anObject.
	self should: [aProxy includesSubString: 'jeje'] raise: MessageNotUnderstood.
	] ensure: [ aProxy mareaUninstallIfNeeded].

! !

!ObjectProxyTracerTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 3/30/2011 16:43'!
testOneSingleObjectProxyDNU

	| anObject proxy |
	
	anObject := ClassWith1Var new.
	proxy := self installProxyOn: anObject. 
	[self assert: proxy class equals: self proxyClass.
	self assert: proxy mareaIsProxyInstalled.
	
	self should: [proxy zaraza] raise: MessageNotUnderstood.
	

	] ensure: [ proxy mareaUninstallIfNeeded].
! !

!ObjectProxyTracerTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 3/30/2011 16:45'!
testOneSingleObjectProxyUndetstandsDNU
	"The object that we swap does also understand the message doesNotUnderstand. We have to be sure not to loop and to correctly call both doesNotUnderstand:"
	| anObject proxy |
	
	anObject := ClassWithDNU new.
	proxy := self installProxyOn: anObject. 
	
	[self assert: proxy class equals: self proxyClass.
	self assert: proxy mareaIsProxyInstalled.
	self assert: proxy zaraza equals: 'dnu'.
	
	] ensure: [ proxy mareaUninstallIfNeeded].
! !

!Behavior methodsFor: '*Proxies' stamp: 'MarianoMartinezPeck 3/14/2011 22:54'!
hasAllMethodsAsProxies
	^ self allMethods allSatisfy: [:aMethod | aMethod mareaIsProxy  ].
	! !

!Behavior methodsFor: '*Proxies' stamp: 'MarianoMartinezPeck 3/13/2011 22:50'!
methodDict
	methodDict == nil ifTrue: ["self manageMDFault" ^ self emptyMethodDictionary  ].
	^ methodDict! !

!CompiledMethod methodsFor: '*Proxies' stamp: 'MarianoMartinezPeck 11/17/2010 21:58'!
mareaPrintString 
	^ self methodClass asString, '>>', self selector.! !

ProtoObject subclass: #ObjectProxyHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Core'!

!ObjectProxyHandler commentStamp: 'MarianoMartinezPeck 4/1/2011 10:17' prior: 0!
ObjectProxyHandler is the general handler for proxies. It is the expected base handler for all handlers, and expects that the proxy instance received as argument  in all its methods, to be an subclass of AbstractObjectProxy.

AbstractObjectProxy subclasses implement the method #mareaHandler which answers the handler to manage such proxy. Examples:

To create a custom object proxy, create a subclass from AbstractObjectProxy (or use one of the existing ones) and a subclass of ObjectProxyHandler. 
For classes, create a subclass of AbstractClassProxy and a subclass of AbstractClassProxyHandler. 
For compiled methods, create a subclass of AbstractCompiledMethodProxy and a subclass of AbstractCompiledMethodProxyHandler

!

ObjectProxyHandler subclass: #AbstractClassProxyHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Core'!

!AbstractClassProxyHandler commentStamp: 'MarianoMartinezPeck 4/1/2011 11:13' prior: 0!
AbstractClassProxyHandler is the base class for proxies for classes. This class is mostly to mark the API that subclasses should follow (implementing abstracts methods). 

This class expects the proxy object passed as argument to be an instance of a subclass of AbstractClassProxy. 

Subclasses should be created to implement the abstract methods and that way, implement the necessary interception functionality. 

Notice that when using proxies for classes, the user can do #become:   between the target class and the proxy. Example:

myInstance := MyClass new.
MyClass become: MyClassProxy new. 

In this case, the handler, needs to intercept both messages: 

1) normal messages that are sent to the proxy as it were the class. Example:
(Smalltalk at: #MyClass) subclasses.
These messages are intercepted with the normal method:  mareaHandleInterception: aMessage for: aProxy

2) messages to instances (which now their class pointer points to the proxy instance) of the target class.
myInstance foo
These messages are intercepted with a special method: mareaHandleInterception: aMessage for: aProxy receiver: anObject

For more details see class comment of AbstractClassProxy

!

!AbstractClassProxyHandler classSide methodsFor: 'proxy handling' stamp: 'MarianoMartinezPeck 4/1/2011 16:02'!
handleInterception: aMessage for: aProxy receiver: anObject
	^ self subclassResponsibility ! !

!AbstractClassProxyHandler methodsFor: 'seeClassSide' stamp: 'MarianoMartinezPeck 4/1/2011 10:07'!
seeClassSide! !

ObjectProxyHandler subclass: #AbstractCompiledMethodProxyHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Core'!

!AbstractCompiledMethodProxyHandler commentStamp: 'MarianoMartinezPeck 4/1/2011 10:30' prior: 0!
AbstractCompiledMethodProxyHandler is the base class for proxies for compiled methods. This class is mostly to mark the API that subclasses should follow (implementing abstracts methods). 

This class expects the proxy object passed as argument to be an instance of a subclass of AbstractCompiledMethodProxy. 

Subclasses should be created to implement the abstract methods and that way, implement the necessary interception functionality. 

Notice that when using proxies for compiled methods, the user can put a proxy instance in a method dictionary of a class. 
Example:

MyClass methodDict at: #foo put: MyMethodProxy new. 

In this case, the handler, needs to intercept both messages: 

1) normal messages that are sent to the proxy as it were the method. Example:
()MyClass >> #foo) literals. 
These messags are intercepted with the normal method:  mareaHandleInterception: aMessage for: aProxy

2) When the method that the proxy proxifies is executed. Example:
MyClass new foo.
This messages are intercepted with a special method: mareaHandleMethodExecutionFor: aSelector with: arguments in: aReceiver proxy: aProxy 


!

!AbstractCompiledMethodProxyHandler classSide methodsFor: 'proxy handling' stamp: 'MarianoMartinezPeck 4/1/2011 16:02'!
handleMethodExecutionFor: aSelector with: arguments in: aReceiver proxy: aProxy 
	^ self subclassResponsibility ! !

!AbstractCompiledMethodProxyHandler methodsFor: 'seeClassSide' stamp: 'MarianoMartinezPeck 4/1/2011 10:03'!
seeClassSide! !

ObjectProxyHandler subclass: #AbstractMethodWrapperProxyHandler
	instanceVariableNames: ''
	classVariableNames: 'MareaSerializer'
	poolDictionaries: ''
	category: 'Proxies-Examples'!

!AbstractMethodWrapperProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/6/2011 20:06'!
handleInterception: aMessage for: aProxy
	| answer  |
	self mareaLog: 'Message intercepted: ', aMessage selector.
	self handlePreInvocation: aMessage for: aProxy.
	answer := aMessage sendTo: aProxy targetMethod.
	self handlePostInvocation: aMessage for: aProxy.
	self mareaLog: 'The message: ', aMessage selector asString, ' was sent to the original object and answered: ', answer asString.
	^ answer 
! !

!AbstractMethodWrapperProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/6/2011 21:26'!
handleMethodExecutionFor: aSelector with: arguments in: aReceiver proxy: aProxy
	| aResult oldMethod |
	self mareaLog: 'The method ', aSelector, ' is being executed with a receiver instance of ', aReceiver mareaClass name.
	self handlePreExecution: aSelector with: arguments in: aReceiver proxy: aProxy.
	oldMethod := aProxy targetMethod.
	aResult := oldMethod valueWithReceiver: aReceiver arguments: arguments.
	self handlePostExecution: aSelector with: arguments in: aReceiver proxy: aProxy.
	self mareaLog: 'The method ', aSelector, ' was executed with a receiver instance of ', aReceiver mareaClass name.
	^aResult
	! !

!AbstractMethodWrapperProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/6/2011 21:26'!
handlePostExecution: aSelector with: arguments in: aReceiver proxy: aProxy.
	^ self subclassResponsibility
	! !

!AbstractMethodWrapperProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/6/2011 20:06'!
handlePostInvocation: aMessage for: aProxy.
	^ self subclassResponsibility ! !

!AbstractMethodWrapperProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/6/2011 21:26'!
handlePreExecution: aSelector with: arguments in: aReceiver proxy: aProxy.
	^ self subclassResponsibility
	self handlePostExecution: aSelector with: arguments in: aReceiver proxy: aProxy.
! !

!AbstractMethodWrapperProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/6/2011 20:06'!
handlePreInvocation: aMessage for: aProxy.
	^ self subclassResponsibility 

! !

!AbstractMethodWrapperProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/6/2011 20:23'!
installFor: aMethodReference andProxy: aCompiledMethod
	| oldMethod oldClass selector |
	oldClass := aMethodReference actualClass. 
	selector := aMethodReference  selector. 
	"Right now, we throw an error if the selector is not found in the class' methodDict. However, in a future, other users may need additional messages so that not only searches de selector in the class's methodDict but also in all the hierarchy. In additio, someone may also need that the proxy is installed in all the implemenetations found in all the hierarchy."
	[oldMethod := oldClass methodDict at: selector ifAbsent: [SelectorNotFoundError signal: ('The selector: ', aMethodReference  selector asString, 
		' was not found in the class: ', aMethodReference actualClass name asString)].
	"The other alternative is being able to create proxies for profies, but the proxy 
	handlers expect a CompiledMethod and not a proxy."
		oldMethod mareaIsProxy.
	] whileTrue: [oldMethod mareaUninstall].
	"oldMethod methodClass superclass isNil ifTrue: [self mareaLog: 'A CompiledMethod cannot be swapped out if the superclass of the class where it is installed is not a subclass of at least ProtoObject. Method: ', oldMethod mareaPrintString].    "
	self mareaLog: 'The method ', aMethodReference mareaPrintString, ' is being wrapped'.
	aCompiledMethod targetMethod: oldMethod.   
	oldClass methodDict at: selector put: aCompiledMethod.
	self mareaLog: 'The method ', aMethodReference mareaPrintString, ' was wrapped'.
	! !

!AbstractMethodWrapperProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/6/2011 20:03'!
uninstallForProxy: aProxy
	^ self uninstallForProxy: aProxy ifNotInstalled: [ ProxyIsNotInstalled signal: 'The proxy is not installed. '].! !

!AbstractMethodWrapperProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/6/2011 20:25'!
uninstallForProxy: aProxy ifNotInstalled: errorBLock
	| oldMethod |
	self mareaLog: 'Starting to uninstall.'.
	aProxy mareaIsProxyInstalled ifFalse: [ 
		self mareaLog: 'The proxy is not installed'.
		errorBLock value ].
	self mareaLog: 'Trying to load.'.
	oldMethod :=  aProxy targetMethod.
	self mareaLog: 'The object ', oldMethod name, ' was loaded form file.'.
	
	oldMethod methodClass  methodDict at: (oldMethod selector) put: oldMethod.
	
	self mareaLog: 'The Proxy for ', oldMethod mareaPrintString, ' was uninstalled.'.
	aProxy mareaMarkAsUninstalled.
	^ oldMethod.  
! !

!AbstractMethodWrapperProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/6/2011 20:03'!
uninstallIfNeeded: aProxy
	^ [self uninstallForProxy: aProxy] on: ProxyIsNotInstalled do: [].! !

!AbstractMethodWrapperProxyHandler methodsFor: 'seeClassSide' stamp: 'MarianoMartinezPeck 4/6/2011 20:03'!
seeClassSide! !

AbstractMethodWrapperProxyHandler subclass: #LoggerMethodWrapperProxyHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Examples'!

!LoggerMethodWrapperProxyHandler classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 4/6/2011 21:27'!
handlePostExecution: aSelector with: arguments in: aReceiver proxy: aProxy.
	Transcript show: '------------------------------------'; cr.
	Transcript show: 'This is the postExecution for message: ', aSelector ; cr.
	Transcript show: '------------------------------------'; cr.

! !

!LoggerMethodWrapperProxyHandler classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 4/6/2011 20:53'!
handlePostInvocation: aMessage for: aProxy.
	Transcript show: '------------------------------------'; cr.
	Transcript show: 'This is the preLoadInvocaton for message: ', aMessage selector; cr.
	Transcript show: '------------------------------------'; cr.

! !

!LoggerMethodWrapperProxyHandler classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 4/6/2011 21:27'!
handlePreExecution: aSelector with: arguments in: aReceiver proxy: aProxy.
	Transcript show: '------------------------------------'; cr.
	Transcript show: 'This is the preExecution for message: ', aSelector ; cr.
	Transcript show: '------------------------------------'; cr.

! !

!LoggerMethodWrapperProxyHandler classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 4/6/2011 20:53'!
handlePreInvocation: aMessage for: aProxy.
	Transcript show: '------------------------------------'; cr.
	Transcript show: 'This is the preLoadInvocaton for message: ', aMessage selector; cr.
	Transcript show: '------------------------------------'; cr.

! !

!ObjectProxyHandler classSide methodsFor: 'proxy handling' stamp: 'MarianoMartinezPeck 4/1/2011 16:00'!
handleInterception: aMessage for: aProxy
	^ self subclassResponsibility ! !

!ObjectProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
installFor: anObject andProxy: aProxy
	^ self subclassResponsibility 
! !

!ObjectProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
uninstallForProxy: aProxy
	^ self uninstallForProxy: aProxy ifNotInstalled: [ ProxyIsNotInstalled signal: 'The proxy is not installed. '].! !

!ObjectProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
uninstallForProxy: aProxy ifNotInstalled: errorBLock
	^ self subclassResponsibility ! !

!ObjectProxyHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
uninstallIfNeeded: aProxy
	^ [self uninstallForProxy: aProxy] on: ProxyIsNotInstalled do: [].! !

!ObjectProxyHandler methodsFor: 'seeClassSide' stamp: 'MarianoMartinezPeck 3/13/2011 12:20'!
seeClassSide! !

ObjectProxyHandler subclass: #ObjectProxySwapperHandler
	instanceVariableNames: ''
	classVariableNames: 'MareaSerializer'
	poolDictionaries: ''
	category: 'Proxies-Swapping'!

!ObjectProxySwapperHandler commentStamp: 'MarianoMartinezPeck 4/1/2011 15:34' prior: 0!
ObjectProxySwapperHandler is the handler for object proxy swappers.

So...the general proxies handling for swapping is the following: there are two objects: the proxy and the target object. The target object can be a regular object, a class or a compiled method.  When you want to proxify any of them, a proxy instance is created from the according class. 

When a proxy is installed on a target object by the handler, the target is serialized and written into a file on disk. The handler does a #become between the proxy and the target. In the proxy, the filename is stored, and when the proxy receives its first message, it intercepts it, and then the handler takes cares of uninstalling the proxy. Uninstalling means load back the original object from disk, deserialize it and do a #become: with the proxy.

For more details look ObjectProxySwapperTest.
!

ObjectProxySwapperHandler subclass: #ClassProxySwapperHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Swapping'!

!ClassProxySwapperHandler commentStamp: 'MarianoMartinezPeck 4/1/2011 15:35' prior: 0!
ClassProxySwapperHandler  is the handler of class proxy swappers.

!

!ClassProxySwapperHandler classSide methodsFor: 'proxy handling' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
handleInterception: aMessage for: aProxy
	| oldClass answer |
	"self mareaCheckLoop."
	self mareaLog: aProxy mareaClass name, ' cannotInterpret: ', aMessage selector.
	oldClass := self uninstallForProxy: aProxy.
	self mareaLog: 'The message will be sent to the original object'.
	answer := aProxy perform: aMessage selector withArguments: aMessage arguments.
	self mareaLog: 'The message was sent to the original object'.
	^ answer.! !

!ClassProxySwapperHandler classSide methodsFor: 'proxy handling' stamp: 'MarianoMartinezPeck 4/1/2011 16:02'!
handleInterception: aMessage for: aProxy receiver: anObject
	| answer |
	self mareaLog: 'cannotInterpret: ', aMessage selector.
	self uninstallForProxy: aProxy.
	self mareaLog: 'The message will be sent to the original object'.
	answer := aMessage sendTo: anObject.
	self mareaLog: 'The message was sent to the original object'.
	^ answer 
! !

!ClassProxySwapperHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 6/28/2011 16:46'!
installFor: aClass andProxy: aProxy
	| className  |
	"There is a problem with compact classes since their instances, do not have a class pointer, and hence, the #become: doesn't update those pointers to point to my proxy. So....nothing to do."	
	aClass indexIfCompact isZero ifFalse: [ CannotProxifyCompactClasses new signal: 'It is not possible to create class proxies for compact classes' ].

	["The other alternative is being able to create proxies for profies, but the proxy 
	handlers expect a Class and not a proxy."
	aClass mareaIsProxy.
	] whileTrue: [aClass mareaUninstall].

	className := aClass name.
	self mareaLog: 'The class ', aClass name, ' is being wrapped'. 
	self writeObjectOnDisk: aClass andProxy: aProxy.
	aClass becomeForward:  aProxy.
	"and the become should be:  aClass becomeForward: aProxy   so that nobody else points to the class and it can be garbage collected"
	"remember that I use cannotInterpret:   so I cannot send ANY message to the proxy because otherwise it will be uninstalled!!"
	self mareaLog: 'The class ', className , ' was wrapped'.
	! !

!ClassProxySwapperHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 7/6/2011 10:24'!
uninstallForProxy: aProxy ifNotInstalled: errorBLock
	| oldObject oldMetaclass hay |
	
	self mareaLog: 'Starting to uninstallXXXXXXX.'.
	aProxy mareaIsProxyInstalled ifFalse: [ 
		self mareaLog: 'The proxy is not installed'.
		errorBLock value ].
	
	self mareaLog: 'Trying to load file named: ', aProxy mareaFileName asString.
	oldObject := self loadObjectFor: aProxy.
	self mareaLog: 'The object ', oldObject name, ' was loaded form file.'.
	
	"These 2 things should actually be done after the #becomeForward: . However, this is not possible because while doing the #becomeForward: all references will be updated to point to the original object. Therefore, there is NO way to get a reference to the proxy after the become. So we do it here but it is not completly correct becasue we are marking a proxy as uninstalled even before it is really done. For example, if there is an error during or after the become, the proxy will be marked as uninstalled even if it was not."
	self removeFile: aProxy mareaFileName.
	aProxy mareaMarkAsUninstalled.

	
	"This has to be a #becomeForward: and NOT a #become. This is because in the object graph brought from disk there can be objects that point of the root, anObject. In such case if we do a #become: those objects will be updated to point to the proxy, what we do not want. An example is when swapping a class. Taking a  class as the root, when loading it back, there will be its metaclass which points to it, and also the literals in the compiled methods will be pointing to the class. If we do a #become:  such objects will point to the proxy. Because of all this, the desired behavior is #becomeForward: . However, we cannot send any message to a proxy because it will be intercepted. Hence, we directly use the implementation of #becomeForward: which luckely uses a primitive and doesnt send any message to the objects."
	oldMetaclass := Metaclass allInstances detect: [:each | (each theNonMetaClass == aProxy ) ] ifNone: []. 		
	oldMetaclass isNil 
		ifTrue: [
			(Array with: aProxy) elementsForwardIdentityTo: (Array with: oldObject).]
		ifFalse: [
			(Array with: aProxy with: oldMetaclass) elementsForwardIdentityTo: (Array with: oldObject with: oldObject class).].
		
	"now oldObject points to the proxy and aProxy points to the old object"
	self mareaLog: 'The Proxy for ', aProxy name, ' was uninstalled.'.
	^ aProxy.  ! !

!ClassProxySwapperHandler classSide methodsFor: 'serialization' stamp: 'MarianoMartinezPeck 8/3/2011 17:01'!
writeObjectOnDisk: aClass andProxy: aProxy
	| refStream fileName serializedObjects |
	fileName := aProxy mareaDefineFileNameFor: aClass.
	[ fileName := fileName , '.swap' ] doWhileTrue: [ FileDirectory default isAFileNamed: fileName ].	"all this thing of the result should be removed when the class serialization is working"
	serializedObjects := self serializer serializeAndWriteClass: aClass to: fileName.
	aProxy mareaStore: fileName serializedObjects: serializedObjects.
	self mareaLog: 'The class ' , aClass mareaPrintString , ' was written on file named: ' , fileName! !

!ClassProxySwapperHandler methodsFor: 'seeClassSide' stamp: 'MarianoMartinezPeck 3/31/2011 17:15'!
seeClassSide! !

ObjectProxySwapperHandler subclass: #CompiledMethodProxySwapperHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Swapping'!

!CompiledMethodProxySwapperHandler commentStamp: 'MarianoMartinezPeck 4/1/2011 15:35' prior: 0!
CompiledMethodProxySwapperHandler  is the handler of CompiledMethod proxy swappers.

!

!CompiledMethodProxySwapperHandler classSide methodsFor: 'proxy handling' stamp: 'MarianoMartinezPeck 4/26/2011 12:17'!
handleMethodExecutionFor: aSelector with: arguments in: aReceiver proxy: aProxy
	| aResult oldMethod |
	"self checkLoop."
	self mareaLog: 'The method ', aSelector, ' is being executed with a receiver instance of ', aReceiver mareaClass name.
	oldMethod := self uninstallForProxy: aProxy.
	aResult := oldMethod valueWithReceiver: aReceiver arguments: arguments.
	self mareaLog: 'The method ', aSelector, ' was executed with a receiver instance of ', aReceiver mareaClass name.
	^aResult
	! !

!CompiledMethodProxySwapperHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 5/2/2011 18:26'!
installFor: aMethodReference andProxy: anObject
	| oldMethod oldClass selector |
	oldClass := aMethodReference actualClass. 
	selector := aMethodReference  selector. 
	"Right now, we throw an error if the selector is not found in the class' methodDict. However, in a future, other users may need additional messages so that not only searches de selector in the class's methodDict but also in all the hierarchy. In additio, someone may also need that the proxy is installed in all the implemenetations found in all the hierarchy."
	[oldMethod := oldClass methodDict at: selector ifAbsent: [SelectorNotFoundError signal: ('The selector: ', aMethodReference  selector asString, 
		' was not found in the class: ', aMethodReference actualClass name asString)].
	"The other alternative is being able to create proxies for profies, but the proxy 
	handlers expect a CompiledMethod and not a proxy."
		oldMethod mareaIsProxy.
	] whileTrue: [oldMethod mareaUninstall].
	"oldMethod methodClass superclass isNil ifTrue: [self mareaLog: 'A CompiledMethod cannot be swapped out if the superclass of the class where it is installed is not a subclass of at least ProtoObject. Method: ', oldMethod mareaPrintString].    "
	self mareaLog: 'The method ', aMethodReference mareaPrintString, ' is being wrapped'.
	self writeObjectOnDisk: oldMethod andProxy: anObject.
	oldClass methodDict at: selector put: anObject.
	self mareaLog: 'The method ', aMethodReference mareaPrintString, ' was wrapped'.
	! !

!CompiledMethodProxySwapperHandler classSide methodsFor: 'constants' stamp: 'MarianoMartinezPeck 11/25/2010 10:11'!
loopTreshhold
	^ 10! !

!CompiledMethodProxySwapperHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 5/6/2011 16:44'!
uninstallForProxy: aProxy ifNotInstalled: errorBLock
	| oldMethod |
	self mareaLog: 'Starting to uninstall.'.
	aProxy mareaIsProxyInstalled ifFalse: [ 
		self mareaLog: 'The proxy is not installed'.
		errorBLock value ].
	self mareaLog: 'Trying to load file named: ', aProxy mareaFileName asString.
	oldMethod := self loadObjectFor: aProxy.
	self mareaLog: 'The object ', oldMethod mareaPrintString , ' was loaded form file.'.
	
	oldMethod methodClass  methodDict at: (oldMethod selector) put: oldMethod.
	self mareaLog: 'The Proxy for ', oldMethod mareaPrintString, ' was uninstalled.'.
	self removeFile: aProxy mareaFileName.
	aProxy mareaMarkAsUninstalled.
	^ oldMethod.  
! !

!CompiledMethodProxySwapperHandler methodsFor: 'seeClassSide' stamp: 'MarianoMartinezPeck 11/25/2010 17:02'!
seeClassSide! !

!ObjectProxySwapperHandler classSide methodsFor: 'proxy handling' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
handleInterception: aMessage for: aProxy
	| answer oldObject |
	self mareaLog: 'cannotInterpret: ', aMessage selector.
	oldObject := self uninstallForProxy: aProxy.
	self mareaLog: 'The message will be sent to the original object'.
	answer := aMessage sendTo: oldObject.
	self mareaLog: 'The message: ', aMessage selector asString, ' was sent to the original object'.
	^ answer 
! !

!ObjectProxySwapperHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 8/2/2011 17:46'!
installFor: anObject andProxy: aProxy
	| objectName  |
	["The other alternative is being able to create proxies for profies, but the proxy 
	handlers expect a Class and not a proxy."
	anObject mareaIsProxy.
	] whileTrue: [anObject mareaUninstall].

	"If an object is global, the serializer will just store the global name. At uninstalling, if we do a become between the proxy and the imported objects we will have problems because the imported object will be what it is in the Smalltalk globals, which is also the proxy. So what happens is that we will never be able to uninstall correctly the proxy. For this reason, global objects cannot be allowed as root. "
	(self isGlobalObject: anObject) ifTrue: [GlobalObjectsCannotBeRoots signal: 'Global objects cannot be roots.' ]. 

	objectName := anObject mareaBasicName.
	self mareaLog: 'The object ', objectName , ' is being wrapped'. 
	self writeObjectOnDisk: anObject andProxy: aProxy.
	anObject becomeForward: aProxy.

	self mareaLog: 'The object ', objectName , ' was wrapped'.
! !

!ObjectProxySwapperHandler classSide methodsFor: 'private' stamp: 'MarianoMartinezPeck 7/15/2011 10:30'!
isGlobalObject: anObject
	self flag: #fixit.
	"there are more globals than just classes and traits."
	^ anObject isBehavior or: [anObject isTrait]
	! !

!ObjectProxySwapperHandler classSide methodsFor: 'serialization' stamp: 'MarianoMartinezPeck 8/6/2011 11:50'!
loadObjectFor: aProxy
	^ self serializer unserializeObjectFrom: aProxy mareaFileName serializedObjects: aProxy mareaSerializedObjects rootProxy: aProxy
	! !

!ObjectProxySwapperHandler classSide methodsFor: 'private' stamp: 'MarianoMartinezPeck 4/1/2011 16:07'!
removeFile: aFileName
	FileDirectory default deleteFileNamed: aFileName
	
! !

!ObjectProxySwapperHandler classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/12/2011 15:41'!
serializer
	MareaSerializer isNil ifTrue: [MareaSerializer := FuelMareaSerializer new].
	^ MareaSerializer! !

!ObjectProxySwapperHandler classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 7/6/2011 11:40'!
serializer: aMareaSerializer
	MareaSerializer := aMareaSerializer! !

!ObjectProxySwapperHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
uninstallForProxy: aProxy
	^ self uninstallForProxy: aProxy ifNotInstalled: [ ProxyIsNotInstalled signal: 'The proxy is not installed. '].! !

!ObjectProxySwapperHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 7/15/2011 10:53'!
uninstallForProxy: aProxy ifNotInstalled: errorBLock
	| oldObject |
	
	self mareaLog: 'Starting to uninstall.'.
	aProxy mareaIsProxyInstalled ifFalse: [ 
		self mareaLog: 'The proxy is not installed'.
		errorBLock value ].
	self mareaLog: 'Trying to load file named: ', aProxy mareaFileName asString.
	oldObject := self loadObjectFor: aProxy.
	self mareaLog: 'The object was loaded form file.'.
	
	"These 2 things should actually be done after the #becomeForward: . However, this is not possible because while doing the #becomeForward: all references will be updated to point to the original object. Therefore, there is NO way to get a reference to the proxy after the become. So we do it here but it is not completly correct becasue we are marking a proxy as uninstalled even before it is really done. For example, if there is an error during or after the become, the proxy will be marked as uninstalled even if it was not."
	self removeFile: aProxy mareaFileName.
	aProxy mareaMarkAsUninstalled.
	
	"This has to be a #becomeForward: and NOT a #become. This is because in the object graph brought from disk there can be objects that point of the root, anObject. In such case if we do a #become: those objects will be updated to point to the proxy, what we do not want. An example is when swapping a class. Taking a  class as the root, when loading it back, there will be its metaclass which points to it, and also the literals in the compiled methods will be pointing to the class. If we do a #become:  such objects will point to the proxy. Because of all this, the desired behavior is #becomeForward: . However, we cannot send any message to a proxy because it will be intercepted. Hence, we directly use the implementation of #becomeForward: which luckely uses a primitive and doesnt send any message to the objects."
	(Array with: aProxy)
		elementsForwardIdentityTo:
			(Array with: oldObject).

	"now oldObject points to the proxy and aProxy points to the old object"
	self mareaLog: 'The Proxy for ', aProxy mareaBasicName, ' was uninstalled.'.
	^ aProxy.  
! !

!ObjectProxySwapperHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
uninstallIfNeeded: aProxy
	^ [self uninstallForProxy: aProxy] on: ProxyIsNotInstalled do: [].! !

!ObjectProxySwapperHandler classSide methodsFor: 'serialization' stamp: 'MarianoMartinezPeck 8/2/2011 10:52'!
writeObjectOnDisk: anObject andProxy: aProxy
	| refStream fileName serializedObjects |
	fileName := aProxy mareaDefineFileNameFor: anObject.
	[ fileName := fileName , '.swap' ] doWhileTrue: [ FileDirectory default isAFileNamed: fileName ].
	serializedObjects := self serializer serializeAndWriteObject: anObject to: fileName.
	aProxy mareaStore: fileName serializedObjects: serializedObjects.
	self mareaLog: 'The object ' , anObject mareaBasicName , ' was written on file named: ' , fileName! !

!ObjectProxySwapperHandler methodsFor: 'seeClassSide' stamp: 'MarianoMartinezPeck 3/30/2011 15:25'!
seeClassSide! !

ObjectProxyHandler subclass: #ObjectProxyTracerHandler
	instanceVariableNames: ''
	classVariableNames: 'MareaSerializer'
	poolDictionaries: ''
	category: 'Proxies-Examples'!

!ObjectProxyTracerHandler commentStamp: 'MarianoMartinezPeck 4/1/2011 15:48' prior: 0!
ObjectProxyTracerHandler  is a concrete subclass of ObjectProxyHandler. This is a simple-fodwarder handler that all it does is to forward the messages to the target object. 

Here is an example (open a Transcript to see the results):

| today proxy |
MareaLogger changeToTranscriptLogger.
today := Date today.
proxy := ObjectProxyWithTarget installOn: today  withHandler: ObjectProxyTracerHandler.
proxy month. 
proxy day.
proxy name.
proxy mareaUninstall.

!

!ObjectProxyTracerHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/1/2011 16:00'!
handleInterception: aMessage for: aProxy
	| answer  |
	self mareaLog: 'Message forwarded: ', aMessage selector.
	self mareaLog: 'The message will be sent to the original object'.
	answer := aMessage sendTo: aProxy target.
	self mareaLog: 'The message: ', aMessage selector asString, ' was sent to the original object and answered: ', answer asString.
	^ answer 
! !

!ObjectProxyTracerHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 7/12/2011 13:38'!
installFor: anObject andProxy: aProxy
	| objectName  |
	["The other alternative is being able to create proxies for profies, but the proxy 
	handlers expect a Class and not a proxy."
	anObject mareaIsProxy.
	] whileTrue: [anObject mareaUninstall].

	objectName := anObject mareaBasicName.
	self mareaLog: 'The object ', objectName , ' is being wrapped'. 
	aProxy target: anObject.
	self mareaLog: 'The object ', objectName , ' was wrapped'.
! !

!ObjectProxyTracerHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
uninstallForProxy: aProxy
	^ self uninstallForProxy: aProxy ifNotInstalled: [ ProxyIsNotInstalled signal: 'The proxy is not installed. '].! !

!ObjectProxyTracerHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
uninstallForProxy: aProxy ifNotInstalled: errorBLock
	
	self mareaLog: 'Starting to uninstall.'.
	aProxy mareaIsProxyInstalled ifFalse: [ errorBLock value ].
	aProxy mareaMarkAsUninstalled.
	self mareaLog: 'The Proxy was uninstalled.'.
	^ aProxy.  
! !

!ObjectProxyTracerHandler classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
uninstallIfNeeded: aProxy
	^ [self uninstallForProxy: aProxy] on: ProxyIsNotInstalled do: [].! !

!ObjectProxyTracerHandler methodsFor: 'seeClassSide' stamp: 'MarianoMartinezPeck 3/30/2011 15:57'!
seeClassSide! !

!ProtoObject methodsFor: '*Proxies' stamp: 'MarianoMartinezPeck 10/20/2010 11:33'!
class
	"Primitive. Answer the object which is the receiver's class. Essential. See 
	Object documentation whatIsAPrimitive."

	<primitive: 111>
	self primitiveFailed! !

!ProtoObject methodsFor: '*Proxies' stamp: 'MarianoMartinezPeck 11/17/2010 19:53'!
mareaClass
	"Primitive. Answer the object which is the receiver's class. Essential. See 
	Object documentation whatIsAPrimitive."

	<primitive: 111>
	self primitiveFailed! !

!MethodReference methodsFor: '*Proxies' stamp: 'MarianoMartinezPeck 11/28/2010 20:21'!
mareaPrintString
	^ self actualClass name, ' >> #', methodSymbol! !

!Object methodsFor: '*Proxies' stamp: 'MarianoMartinezPeck 7/12/2011 13:35'!
mareaBasicName
	| title |
	title := self class name.
	^ String streamContents: [:aStream| 
		aStream
			nextPutAll: (title first isVowel ifTrue: ['an '] ifFalse: ['a ']);
			nextPutAll: title
		
	]
! !

!Object methodsFor: '*Proxies' stamp: 'MarianoMartinezPeck 3/14/2011 19:42'!
mareaPrintString
	^ self printString! !

Object subclass: #ObjectProxyInterceptor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Core'!

!ObjectProxyInterceptor commentStamp: 'MarianoMartinezPeck 4/1/2011 10:38' prior: 0!
ObjectProxyInterceptor is the superclass in the AbstractObjectProxy that takes care about the method interception and consequently delegate accordly to the ObjectProxyHandler. 

The interception happens due to the methodDict of the subclass ObjectProxyTrap being nil. When the VM finds during the method lookup, that a methodDict is nil, it sends the message #cannotInterpret: to the receiver, but starting the method lookup in the superclass of the class whose methodDict was nil.   

This class is transparent for the final user of the framework. !

!ObjectProxyInterceptor methodsFor: 'intercepting' stamp: 'MarianoMartinezPeck 10/4/2011 17:25'!
cannotInterpret: aMessage
	| proxy context |

"	context := thisContext sender.
	10 timesRepeat: [
		Transcript show: 'Sender: ', context  method selector; cr.
		 context := context sender.
		].
	"
		
	proxy := aMessage lookupClass.
	"If the lookUp class is a proxy, it means that it was a class proxy. Otherwise, lookupClass should be a class of a proxy, which is not a proxy"
	proxy mareaIsProxy
		ifTrue:   [
			^ proxy mareaHandler handleInterception: aMessage for: proxy receiver: self
		]
	 	ifFalse: [
			self mareaIsProxyInstalled 
				ifTrue: [^ self mareaHandler handleInterception: aMessage for: self]
				ifFalse: [
					aMessage lookupClass: thisContext method methodClass.
					"I could replace this by aMessage lookupClass: ObjectProxyInterceptor.
					but I prefer not to hardcode the class"
					^ self mareaPerform: aMessage selector withArguments: aMessage arguments  inSuperclass: aMessage lookupClass
				]
			]! !

ObjectProxyInterceptor subclass: #ObjectProxyTrap
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Core'!

!ObjectProxyTrap commentStamp: 'MarianoMartinezPeck 4/1/2011 10:40' prior: 0!
ObjectProxyInterceptor is the superclass in the AbstractObjectProxy and all its duty is to be the method interception trap. To be a trap in this case, means to have the methodDict in nil. 

The interception happens due to the methodDict of the subclass ObjectProxyTrap being nil. When the VM finds during the method lookup, that a methodDict is nil, it sends the message #cannotInterpret: to the receiver, but starting the method lookup in the superclass of the class whose methodDict was nil. This is why #cannotInterpret: is implemented in the superclass ObjectProxyInterceptor.  

This class is transparent for the final user of the framework.!

ObjectProxyTrap subclass: #AbstractObjectProxy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Core'!

AbstractObjectProxy class
	instanceVariableNames: 'IsRunnintTests'!

!AbstractObjectProxy commentStamp: 'MarianoMartinezPeck 4/1/2011 11:30' prior: 0!
AbstractObjectProxy is the base class for all proxies of this toolbox. The responsability of this class, and its subclasses, is to act as a proxy. But this means, only taking care about the message interception and what/how is stored in a proxy. For example, one proxy implementation can store a targe object, and another one can stored the filename because the proxified object is moved to disk.

The proxy DO NOT take care about the handling of message interception, it just intercepts and forward them to a handler, specified by the answer of #mareaHandler. This method should answer a subclass of ObjectProxyHandler. It is up to the proxy implementation to decide wether such handler should be an instance of the proxy, or by hardcoded. For example, ObjectProxyWithTarget has an instance variable with the handler, this means each proxy instance can have its own handler. On the other hand, all instances of AbstractObjectProxySwapper answer ObjectProxySwapperHandler.

Notice that the same proxy can use different handlers or vice-versa (check the classes FilenameCompiledMethodProxySwapper and NumberAsFilenameCompiledMethodProxySwapper as an example).

It is also very important to understand the problem of method collisions. If you create a subclass and implements the message #foo, and if the target object does also understand #foo, then since the proxy understands #foo, it is not forwarded to the target object, and instead it answers its own implementation. This can be a problem. What we did is to use a specific prefix (marea) in our proxy hierachy. If you subclass, we recommend you to take the same approach of be very careful wiht the method you implment.

Concrete subclasses of his class are for regular objects (not methods or classes). For an example, take a look to ObjectProxyWithTarget and ObjectProxyTracerHandler.!

AbstractObjectProxy class
	instanceVariableNames: 'IsRunnintTests'!

AbstractObjectProxy subclass: #AbstractClassProxy
	instanceVariableNames: 'superclass methodDict format'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Core'!

!AbstractClassProxy commentStamp: 'MarianoMartinezPeck 4/1/2011 11:12' prior: 0!
AbstractClassProxy is the base class for all proxies for classes. Notice that it is not only that we can create proxies for classes, but instead, we can use proxies AS classes. This mean that when using proxies for classes, the user can do #become:   between the target class and the proxy. Example:

myInstance := MyClass new.
MyClass become: MyClassProxy new. 

Now, imagine you do:

myInstance foo

The VM will start to do the method lookup, and it will check in the class of myInstance whether the method is included or not in the method dictionary. But....we do not have a class object, but a proxy instance instead. Hence, the VM will crash because it expects that a class has certain shape. In order to be able to use proxies as classes, we need that a proxy instance has the "shape" of a class. This is why this proxy has these 3 instance variables: superclass, methodDict and format.

But all that is transparent for the user, which only needs to subclass from AbstractClassProxy and associate a handler which must be a subclass of AbstractClassProxyHandler.

For more details see class comment of AbstractClassProxyHandler!

!AbstractClassProxy methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 3/31/2011 18:40'!
initialize
	superclass := ObjectProxyInterceptor.
	methodDict := nil.
	format := Object format! !

AbstractClassProxy subclass: #AbstractClassProxySwapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Swapping'!

!AbstractClassProxySwapper commentStamp: 'MarianoMartinezPeck 4/1/2011 15:22' prior: 0!
AbstractClassProxySwapper is the base for different possible class proxies for swapping. For the moment there is only one concrete subclass, ClassProxySwapper, but there can be more.

This class is almost for marking the API and that the user know what is needed if it needs to subclass. 

This proxy uses a subclass of AbstractClassProxyHandler as handler.

For more details check ClassProxySwapperTest.!

!AbstractClassProxySwapper methodsFor: 'class membership' stamp: 'MarianoMartinezPeck 4/26/2011 01:01'!
class
	((self mareaClass isRunningTest) or: [self mareaIsProxyInstalled not])
		ifTrue: [^ self mareaClass ]
		ifFalse: [ ^ self mareaUninstall class ].
	! !

!AbstractClassProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/31/2011 18:43'!
mareaDefineFileNameFor: anObject
	^ self subclassResponsibility! !

!AbstractClassProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/31/2011 18:43'!
mareaFileName
	^ self subclassResponsibility ! !

!AbstractClassProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 3/31/2011 18:42'!
mareaHandler
	^ ClassProxySwapperHandler! !

!AbstractClassProxySwapper methodsFor: 'testing' stamp: 'MarianoMartinezPeck 3/31/2011 18:43'!
mareaIsProxyInstalled
	^ self subclassResponsibility ! !

!AbstractClassProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 3/31/2011 18:43'!
mareaMarkAsUninstalled
	! !

!AbstractClassProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 8/2/2011 10:53'!
mareaStore: aFileName serializedObjects: serializedObjects
	! !

!AbstractClassProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 3/31/2011 18:44'!
mareaStoreFileNameIfNecessary: aFileName result: aResult
	! !

AbstractClassProxySwapper subclass: #ClassProxySwapper
	instanceVariableNames: 'fileName weakSerializedObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Swapping'!

!ClassProxySwapper commentStamp: 'MarianoMartinezPeck 4/1/2011 15:04' prior: 0!
ClassProxySwapper is a concrete subclass of AbstractClassProxySwapper, which respobsavility is to create class proxies for swapping. This proxy does not store the target object but instead its filename (because it will be written on disk by the handler).

So...when an instance of ClassProxySwapper is installed on a target object by the handler, the target is serialized and written into a file on disk. The handler does a #become between the proxy and the target. In the proxy, the filename is stored, and when the proxy receives its first message, it intercepts it, and then the handler takes cares of uninstalling the proxy. Uninstalling means load back the origina object from disk, deserialize it and do a #become: with the proxy.

This class defined that the filename is the name of the class. However, another possible class could have the value of a SmallInteger (in order to use less memory). 

For more details, check ClassProxySwapperHandler and ClassProxySwapperTest.

Instance Variables:
	result	<ProtoObject | PseudoContext>
	fileName	<ProtoObject>!

!ClassProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 5/13/2011 16:09'!
isBehavior 
	^ false! !

!ClassProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/31/2011 18:39'!
mareaDefineFileNameFor: aClass
	^ aClass name! !

!ClassProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/31/2011 18:39'!
mareaFileName
	^ fileName! !

!ClassProxySwapper methodsFor: 'testing' stamp: 'MarianoMartinezPeck 3/31/2011 18:39'!
mareaIsProxyInstalled
	^  fileName isNil not! !

!ClassProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 3/31/2011 18:39'!
mareaMarkAsUninstalled
	fileName := nil.! !

!ClassProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 8/3/2011 16:29'!
mareaSerializedObjects
	^ weakSerializedObjects! !

!ClassProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 8/3/2011 16:29'!
mareaStore: aFileName serializedObjects: serializedObjects
	fileName := aFileName.
	weakSerializedObjects := serializedObjects ! !

AbstractObjectProxy subclass: #AbstractCompiledMethodProxy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Core'!

!AbstractCompiledMethodProxy commentStamp: 'MarianoMartinezPeck 4/1/2011 11:24' prior: 0!
AbstractCompiledMethodProxy is the base class for all proxies for compiled methods. Notice that it is not only that we can create proxies for methods, but instead, we can use proxies AS methods. This mean that when using proxies for methods, the user can put a proxy instance in a method dictionary of a class. Example:

MyClass methodDict at: #foo put: MyMethodProxy new. 

In this case, the proxy needs to intercept both messages: 

1) normal messages that are sent to the proxy as it were the method. Example:
()MyClass >> #foo) literals. 
These messags are intercepted with the normal trick of the #cannotInterpret and using ObjectProxyTrap/ObjectProxyInterceptor.

2) When the method that the proxy proxifies is executed. Example:
MyClass new foo.
These messages are intercepted by implementing the method #run: aSelector with: arguments in: aReceiver. When the VM does the method lookup and finds the selector, if the associated compiled method, is not really an instance of CompiledMethod, it sends #run:with:in: to such object.

As you can see, in order to be able to behave as a compiled method, this class implement some methods like #flushCache and #run:with:in:

Users must subclass from AbstractCompiledMethodProxy and should map its handler to a subclass of AbstractCompiledMethodProxyHandler.

For more details see class comment of AbstractCompiledMethodProxyHandler!

!AbstractCompiledMethodProxy classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 12/1/2010 18:07'!
installOn: aClass selector: aSel
	^ (self on: aClass selector: aSel)! !

!AbstractCompiledMethodProxy classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 3/31/2011 10:14'!
on: aMethodReference
	^ super installOn: aMethodReference ! !

!AbstractCompiledMethodProxy classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 12/1/2010 18:07'!
on: aClass selector: aSelector
	^self on: (MethodReference class: aClass selector: aSelector)! !

!AbstractCompiledMethodProxy methodsFor: 'objects as mehods API' stamp: 'MarianoMartinezPeck 4/1/2011 11:38'!
flushCache
	self primFlushCacheIfSelector.
	self primFlushCacheIfMethod! !

!AbstractCompiledMethodProxy methodsFor: 'objects as mehods API' stamp: 'MarianoMartinezPeck 4/1/2011 11:37'!
primFlushCacheIfMethod
	"Tell the interpreter to remove all references to this method from its method lookup cache, if it has one.  This primitive must be called whenever a method is defined or removed.
	NOTE:  Only one of two selective flush methods needs to be used.
	Squeak 2.2 and earlier uses 119 (See Symbol flushCache).
	Squeak 2.3 and later uses 116 (See CompiledMethod flushCache)."

	<primitive: 116>
! !

!AbstractCompiledMethodProxy methodsFor: 'objects as mehods API' stamp: 'MarianoMartinezPeck 4/1/2011 11:37'!
primFlushCacheIfSelector
	"Tell the interpreter to remove all entries with this symbol as a selector from its method lookup cache, if it has one.  This primitive must be called whenever a method is defined or removed.
	NOTE:  Only one of the two selective flush methods needs to be used.
	Squeak 2.3 and later uses 116 (See CompiledMethod flushCache)."

	<primitive: 119>
! !

!AbstractCompiledMethodProxy methodsFor: 'objects as mehods API' stamp: 'MarianoMartinezPeck 4/1/2011 16:02'!
run: aSelector with: arguments in: aReceiver
	^ self mareaHandler handleMethodExecutionFor: aSelector with: arguments in: aReceiver proxy: self
	! !

AbstractCompiledMethodProxy subclass: #AbstractCompiledMethodProxySwapper
	instanceVariableNames: 'weakSerializedObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Swapping'!

!AbstractCompiledMethodProxySwapper commentStamp: 'MarianoMartinezPeck 4/1/2011 15:22' prior: 0!
AbstractCompiledMethodProxySwapper is the base for different possible compiled method proxy for swapping. For the moment there are two classes, which only differ in the way the filename of the swapped out method is stored. 

This proxy uses a subclass of AbstractCompiledMethodProxyHandler as handler.

This class is almost for marking the API and that the user know what is needed if it needs to subclass. 

For more details check AbstractCompiledMethodProxyTest.!

!AbstractCompiledMethodProxySwapper methodsFor: 'class membership' stamp: 'MarianoMartinezPeck 4/26/2011 01:01'!
class
	((self mareaClass isRunningTest) or: [self mareaIsProxyInstalled not])
		ifTrue: [^ self mareaClass ]
		ifFalse: [ ^ self mareaUninstall class ].
	! !

!AbstractCompiledMethodProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/31/2011 10:10'!
mareaDefineFileNameFor: aCompiledMethod
	^ self subclassResponsibility ! !

!AbstractCompiledMethodProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/31/2011 10:11'!
mareaFileName
	^ self subclassResponsibility ! !

!AbstractCompiledMethodProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/31/2011 10:21'!
mareaHandler
	^ CompiledMethodProxySwapperHandler! !

!AbstractCompiledMethodProxySwapper methodsFor: 'testing' stamp: 'MarianoMartinezPeck 3/31/2011 10:11'!
mareaIsProxyInstalled
	^ self subclassResponsibility ! !

!AbstractCompiledMethodProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 3/31/2011 10:11'!
mareaMarkAsUninstalled
	! !

!AbstractCompiledMethodProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 8/3/2011 16:29'!
mareaSerializedObjects
	^ weakSerializedObjects! !

!AbstractCompiledMethodProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 8/3/2011 16:29'!
mareaStore: aFileName serializedObjects: serializedObjects
	weakSerializedObjects := serializedObjects ! !

AbstractCompiledMethodProxySwapper subclass: #FilenameCompiledMethodProxySwapper
	instanceVariableNames: 'fileName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Swapping'!

!FilenameCompiledMethodProxySwapper commentStamp: 'MarianoMartinezPeck 4/1/2011 15:12' prior: 0!
FilenameCompiledMethodProxySwapper is a concrete subclass of AbstractCompiledMethodProxySwapper that stores the proxied compiled method signature as the filename. 

For more details check FilenameCompiledMethodProxySwapperFuelTest or FilenameCompiledMethodProxySwapperReferenceStreamTest.

!

!FilenameCompiledMethodProxySwapper classSide methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 12/13/2010 23:28'!
initialize
	self becomeCompact.! !

!FilenameCompiledMethodProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/30/2011 15:19'!
mareaDefineFileNameFor: aCompiledMethod
	^ aCompiledMethod mareaPrintString! !

!FilenameCompiledMethodProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 11/23/2010 12:05'!
mareaFileName
	^ fileName! !

!FilenameCompiledMethodProxySwapper methodsFor: 'testing' stamp: 'MarianoMartinezPeck 11/24/2010 11:33'!
mareaIsProxyInstalled
	^ fileName isNil not! !

!FilenameCompiledMethodProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 11/24/2010 12:10'!
mareaMarkAsUninstalled
	fileName := nil.! !

!FilenameCompiledMethodProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 8/3/2011 17:52'!
mareaStore: aFileName serializedObjects: serializedObjects
	super mareaStore: aFileName serializedObjects: serializedObjects.
	fileName := aFileName! !

AbstractCompiledMethodProxySwapper subclass: #NumberAsFilenameCompiledMethodProxySwapper
	instanceVariableNames: 'number'
	classVariableNames: 'CurrentNumber'
	poolDictionaries: ''
	category: 'Proxies-Swapping'!

!NumberAsFilenameCompiledMethodProxySwapper commentStamp: 'MarianoMartinezPeck 4/1/2011 15:13' prior: 0!
NumberAsFilenameCompiledMethodProxySwapper  is a concrete subclass of AbstractCompiledMethodProxySwapper that stores a SmallInteger as the filename of the proxified CompiledMethod. 

For more details check NumberAsFilenameCompiledMethodProxySwapperFuelTest or NumberAsFilenameCompiledMethodProxySwapperReferenceStreamTest.

!

!NumberAsFilenameCompiledMethodProxySwapper classSide methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 5/6/2011 15:31'!
initialize
	self becomeCompact.
	CurrentNumber := 0.! !

!NumberAsFilenameCompiledMethodProxySwapper classSide methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 5/6/2011 15:34'!
mareaCurrentNumber
	CurrentNumber isNil ifTrue: [ CurrentNumber := 0 ].
	^ CurrentNumber
! !

!NumberAsFilenameCompiledMethodProxySwapper classSide methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 5/6/2011 15:31'!
mareaNextNumber
	CurrentNumber := self mareaCurrentNumber + 1.
	^ CurrentNumber
! !

!NumberAsFilenameCompiledMethodProxySwapper classSide methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 5/6/2011 15:31'!
mareaResetNumber.
	CurrentNumber := nil! !

!NumberAsFilenameCompiledMethodProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/6/2011 15:31'!
mareaDefineFileNameFor: aCompiledMethod
	number := self class mareaNextNumber.
	^  number asString! !

!NumberAsFilenameCompiledMethodProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/31/2011 10:17'!
mareaFileName
	^ number asString, '.swap'! !

!NumberAsFilenameCompiledMethodProxySwapper methodsFor: 'testing' stamp: 'MarianoMartinezPeck 3/31/2011 10:17'!
mareaIsProxyInstalled
	^ number isNil not! !

!NumberAsFilenameCompiledMethodProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 3/31/2011 10:17'!
mareaMarkAsUninstalled
	number := nil.! !

AbstractCompiledMethodProxy subclass: #MethodWrapperProxy
	instanceVariableNames: 'targetMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Examples'!

!MethodWrapperProxy classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 4/6/2011 20:37'!
installOn: aClass selector: aSel
	^ (self on: aClass selector: aSel)! !

!MethodWrapperProxy classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 4/6/2011 20:37'!
on: aMethodReference
	^ super installOn: aMethodReference ! !

!MethodWrapperProxy classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 4/6/2011 20:37'!
on: aClass selector: aSelector
	^self on: (MethodReference class: aClass selector: aSelector)! !

!MethodWrapperProxy methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 4/6/2011 20:50'!
class
	((self mareaClass isRunningTest) or: [self mareaIsProxyInstalled not])
		ifTrue: [^ self mareaClass ]
		ifFalse: [ ^ self mareaUninstall class ].
	! !

!MethodWrapperProxy methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 4/6/2011 20:35'!
mareaHandler
	^ LoggerMethodWrapperProxyHandler! !

!MethodWrapperProxy methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 4/6/2011 20:48'!
mareaIsProxyInstalled
	^ targetMethod isNil not! !

!MethodWrapperProxy methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 4/6/2011 20:48'!
mareaMarkAsUninstalled
	targetMethod  := nil.! !

!MethodWrapperProxy methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 4/6/2011 20:02'!
targetMethod
	^ targetMethod! !

!MethodWrapperProxy methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 4/6/2011 20:02'!
targetMethod: anObject
	targetMethod := anObject! !

!AbstractObjectProxy classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/31/2011 12:34'!
allInstances
	"Answer a collection of all current instances of the receiver."

	| all inst next |
	all := OrderedCollection new.
	inst := self someInstance.
	[inst == nil]
		whileFalse: [
		next := inst mareaNextInstance.
		inst == all ifFalse: [all add: inst].
		inst := next].
	^ all asArray! !

!AbstractObjectProxy classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/31/2011 12:31'!
allInstancesDo: aBlock 
	"Evaluate the argument, aBlock, for each of the current instances of the 
	receiver.
	
	Because aBlock might change the class of inst (for example, using become:),
	it is essential to compute next before aBlock value: inst."
	| inst next |
	inst := self someInstance.
	[inst == nil]
		whileFalse:
		[
		next := inst mareaNextInstance.
		aBlock value: inst.
		inst := next]! !

!AbstractObjectProxy classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 3/31/2011 10:42'!
installOn: anObject
	| newProxy originalProxy |
	newProxy := self new.
	originalProxy := newProxy mareaInstallFor: anObject.
	ObjectProxyTrap nilMethodDict.
	^ newProxy ! !

!AbstractObjectProxy classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/30/2011 15:11'!
isRunningTest
	IsRunnintTests isNil ifTrue: [IsRunnintTests := true].
	^ IsRunnintTests ! !

!AbstractObjectProxy classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 7/6/2011 15:45'!
mareaAllInstalledProxies
	^ self allInstances select: [:each | each mareaIsProxyInstalled ]
 
! !

!AbstractObjectProxy classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 7/6/2011 15:44'!
mareaAllInstalledProxiesCount
	^ self mareaAllInstalledProxies size
	
 
! !

!AbstractObjectProxy classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 7/6/2011 15:45'!
mareaUninstallIfNeededAllInstances
	self allInstancesDo: [:each | each mareaUninstallIfNeeded ]
 
! !

!AbstractObjectProxy classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 7/6/2011 15:45'!
mareaUninstallIfNeededAllInstancesWithTracing: aLogger
	| counter |
	counter := 0.
	self allInstancesDo: [:instance| 
		(counter \\ 5000) isZero ifTrue: [
			aLogger log: ''.
			aLogger log: SmalltalkImage current vmStatisticsReportString.
			aLogger log: ''.
		]. 
	counter := counter + 1.
	instance mareaUninstallIfNeeded
	].
! !

!AbstractObjectProxy classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 7/18/2011 10:26'!
newForTesting
	| newProxy |
	newProxy := self new.
	ObjectProxyTrap nilMethodDict.
	^ newProxy ! !

!AbstractObjectProxy classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 10/3/2011 16:37'!
newProxy
	| newProxy |
	newProxy := self new.
	"Just in case"
	ObjectProxyTrap nilMethodDict.
	^ newProxy ! !

!AbstractObjectProxy classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 10/2/2011 14:52'!
newProxyFor: anObject 
	| newProxy |
	newProxy := self new.
	"Just in case"
	ObjectProxyTrap nilMethodDict.
	^ newProxy ! !

!AbstractObjectProxy classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/30/2011 15:11'!
runningTest: aBoolean
	IsRunnintTests := aBoolean  ! !

!AbstractObjectProxy methodsFor: '*Proxies-Serializers' stamp: 'MarianoMartinezPeck 10/4/2011 17:06'!
asSetElement
	"Answer an object, which can be put into a Set as element , wrapped
	by one of SetElement instance, if necessary. 
	Default implementation is to answer self"
! !

!AbstractObjectProxy methodsFor: 'class membership' stamp: 'MarianoMartinezPeck 3/30/2011 17:13'!
class
	^ self subclassResponsibility ! !

!AbstractObjectProxy methodsFor: '*Proxies-Serializers' stamp: 'MarianoMartinezPeck 7/18/2011 10:41'!
enclosedSetElement! !

!AbstractObjectProxy methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 3/30/2011 15:11'!
initialize! !

!AbstractObjectProxy methodsFor: '*Proxies-Serializers' stamp: 'MarianoMartinezPeck 10/4/2011 15:38'!
mareaBasicIdentityHash
	"Answer a SmallInteger whose value is related to the receiver's identity.
	This method must not be overridden, except by SmallInteger.
	Primitive. Fails if the receiver is a SmallInteger. Essential.
	See Object documentation whatIsAPrimitive.

	Do not override. Use #identityHash unless you really know what you're doing.'"

	<primitive: 75>
	self primitiveFailed! !

!AbstractObjectProxy methodsFor: '*Proxies-Serializers' stamp: 'MarianoMartinezPeck 7/18/2011 10:43'!
mareaBasicName
	^ 'aProxy'! !

!AbstractObjectProxy methodsFor: 'class membership' stamp: 'MarianoMartinezPeck 3/30/2011 15:37'!
mareaClass
	"Primitive. Answer the object which is the receiver's class. Essential. See 
	Object documentation whatIsAPrimitive."

	<primitive: 111>
	self primitiveFailed! !

!AbstractObjectProxy methodsFor: 'handler' stamp: 'MarianoMartinezPeck 3/30/2011 15:43'!
mareaHandler
	^ self subclassResponsibility ! !

!AbstractObjectProxy methodsFor: '*Proxies-Serializers' stamp: 'MarianoMartinezPeck 10/4/2011 15:39'!
mareaIdentityHash
	"Answer a SmallInteger whose value is related to the receiver's identity.
	This method must not be overridden, except by SmallInteger.
	
	Do not override."
	
	^ self mareaBasicIdentityHash bitShift: 18! !

!AbstractObjectProxy methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
mareaInstallFor: anObject
	self mareaHandler installFor: anObject andProxy: self
	! !

!AbstractObjectProxy methodsFor: 'testing' stamp: 'MarianoMartinezPeck 3/30/2011 15:11'!
mareaIsProxy
	^ true! !

!AbstractObjectProxy methodsFor: 'testing' stamp: 'MarianoMartinezPeck 3/30/2011 15:12'!
mareaIsProxyInstalled
	^ self subclassResponsibility ! !

!AbstractObjectProxy methodsFor: 'logging' stamp: 'MarianoMartinezPeck 3/30/2011 15:11'!
mareaLog: anObject
	MareaLogger logger log: anObject.! !

!AbstractObjectProxy methodsFor: 'updating' stamp: 'MarianoMartinezPeck 3/30/2011 15:12'!
mareaMarkAsUninstalled
	! !

!AbstractObjectProxy methodsFor: 'enumerating' stamp: 'MarianoMartinezPeck 3/31/2011 12:32'!
mareaNextInstance
	"Primitive. Answer the next instance after the receiver in the 
	enumeration of all instances of this class. Fails if all instances have been 
	enumerated. Essential. See Object documentation whatIsAPrimitive."

	<primitive: 78>
	^nil! !

!AbstractObjectProxy methodsFor: 'message handling' stamp: 'MarianoMartinezPeck 3/31/2011 15:11'!
mareaPerform: selector withArguments: argArray inSuperclass: lookupClass
	"NOTE:  This is just like perform:withArguments:, except that
	the message lookup process begins, not with the receivers's class,
	but with the supplied superclass instead.  It will fail if lookupClass
	cannot be found among the receiver's superclasses.
	Primitive. Essential. See Object documentation whatIsAPrimitive."

	<primitive: 100>
	(selector isSymbol)
		ifFalse: [^ self error: 'selector argument must be a Symbol'].
	(selector numArgs = argArray size)
		ifFalse: [^ self error: 'incorrect number of arguments'].
	(self class == lookupClass or: [self class inheritsFrom: lookupClass])
		ifFalse: [^ self error: 'lookupClass is not in my inheritance chain'].
	self primitiveFailed! !

!AbstractObjectProxy methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
mareaUninstall
	^ self mareaHandler uninstallForProxy: self
! !

!AbstractObjectProxy methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 4/1/2011 16:01'!
mareaUninstallIfNeeded
	^ self mareaHandler uninstallIfNeeded: self
! !

AbstractObjectProxy subclass: #AbstractObjectProxySwapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Swapping'!

!AbstractObjectProxySwapper commentStamp: 'MarianoMartinezPeck 4/1/2011 15:24' prior: 0!
AbstractObjectProxySwapper  is the base for different possible object proxy for swapping. 

This class is almost for marking the API and that the user know what is needed if it needs to subclass. 

This proxy uses a subclass of ObjectProxyHandler as handler.

For more details check ObjectProxySwapperTest.!

!AbstractObjectProxySwapper methodsFor: 'class membership' stamp: 'MarianoMartinezPeck 3/30/2011 17:13'!
class
	((self mareaClass isRunningTest) or: [self mareaIsProxyInstalled not])
		ifTrue: [^ self mareaClass ]
		ifFalse: [ ^ self mareaUninstall class ].
	! !

!AbstractObjectProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/30/2011 15:36'!
mareaDefineFileNameFor: anObject
	^ self subclassResponsibility! !

!AbstractObjectProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/30/2011 15:36'!
mareaFileName
	^ self subclassResponsibility ! !

!AbstractObjectProxySwapper methodsFor: 'handler' stamp: 'MarianoMartinezPeck 3/30/2011 15:44'!
mareaHandler
	^ ObjectProxySwapperHandler! !

!AbstractObjectProxySwapper methodsFor: 'testing' stamp: 'MarianoMartinezPeck 3/30/2011 15:36'!
mareaIsProxyInstalled
	^ self subclassResponsibility ! !

!AbstractObjectProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 8/2/2011 10:53'!
mareaStore: aFileName serializedObjects: serializedObjects! !

AbstractObjectProxySwapper subclass: #NumberAsFilenameObjectProxySwapper
	instanceVariableNames: 'number weakSerializedObjects'
	classVariableNames: 'CurrentNumber'
	poolDictionaries: ''
	category: 'Proxies-Swapping'!

!NumberAsFilenameObjectProxySwapper commentStamp: 'MarianoMartinezPeck 4/1/2011 15:16' prior: 0!
NumberAsFilenameObjectProxySwapper  is a concrete subclass of AbstractObjectProxySwapper that stores a SmallInteger as the filename of the proxified CompiledMethod. 

For more details check ObjectProxySwapperTest
!

!NumberAsFilenameObjectProxySwapper classSide methodsFor: 'initialize' stamp: 'MarianoMartinezPeck 5/6/2011 15:30'!
initialize
	self becomeCompact.
	CurrentNumber := 0.! !

!NumberAsFilenameObjectProxySwapper classSide methodsFor: 'initialize' stamp: 'MarianoMartinezPeck 5/6/2011 15:34'!
mareaCurrentNumber
	CurrentNumber isNil ifTrue: [ CurrentNumber := 0 ].
	^ CurrentNumber
! !

!NumberAsFilenameObjectProxySwapper classSide methodsFor: 'initialize' stamp: 'MarianoMartinezPeck 5/6/2011 15:30'!
mareaNextNumber
	CurrentNumber := self mareaCurrentNumber + 1.
	^ CurrentNumber
! !

!NumberAsFilenameObjectProxySwapper classSide methodsFor: 'initialize' stamp: 'MarianoMartinezPeck 5/6/2011 15:30'!
mareaResetNumber.
	CurrentNumber := nil! !

!NumberAsFilenameObjectProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/6/2011 15:30'!
mareaDefineFileNameFor: anObject
	number := self class mareaNextNumber.
	^  number asString! !

!NumberAsFilenameObjectProxySwapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/30/2011 12:16'!
mareaFileName
	^ number asString, '.swap'! !

!NumberAsFilenameObjectProxySwapper methodsFor: 'testing' stamp: 'MarianoMartinezPeck 3/30/2011 12:16'!
mareaIsProxyInstalled
	^ number isNil not! !

!NumberAsFilenameObjectProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 3/30/2011 12:16'!
mareaMarkAsUninstalled
	number := nil.! !

!NumberAsFilenameObjectProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 8/2/2011 10:56'!
mareaSerializedObjects
	^ weakSerializedObjects ! !

!NumberAsFilenameObjectProxySwapper methodsFor: 'updating' stamp: 'MarianoMartinezPeck 8/2/2011 10:54'!
mareaStore: aFileName serializedObjects: serializedObjects
	weakSerializedObjects := serializedObjects ! !

AbstractObjectProxy subclass: #ObjectProxyWithTarget
	instanceVariableNames: 'target handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Examples'!

!ObjectProxyWithTarget commentStamp: 'MarianoMartinezPeck 4/1/2011 15:46' prior: 0!
ObjectProxyWithTarget is a concrete subclass of AbstractObjectProxy, which stores the target object in an instance variable.  Since it is a subclass from  AbstractObjectProxy we can only use it for regular objects. 

Their instances also stores a handler, which should be a subclass of ObjectProxyHandler.  ObjectProxyTracerHandler is a handler that all it does is to fordward the messages to the target object. 

Here is an example (open a Transcript to see the results):

| today proxy |
MareaLogger changeToTranscriptLogger.
today := Date today.
proxy := ObjectProxyWithTarget installOn: today  withHandler: ObjectProxyTracerHandler.
proxy month. 
proxy day.
proxy name.
proxy mareaUninstall.

!

!ObjectProxyWithTarget classSide methodsFor: 'install-uninstall' stamp: 'MarianoMartinezPeck 3/30/2011 16:34'!
installOn: anObject withHandler: aHandler
	| newProxy originalProxy |
	newProxy := self new.
	newProxy handler: aHandler.
	originalProxy := newProxy mareaInstallFor: anObject.
	ObjectProxyTrap nilMethodDict.
	^ newProxy! !

!ObjectProxyWithTarget methodsFor: 'class membership' stamp: 'MarianoMartinezPeck 3/30/2011 17:13'!
class
	((self mareaClass isRunningTest) or: [self mareaIsProxyInstalled not])
		ifTrue: [^ self mareaClass ]
		ifFalse: [ ^ self target class ].! !

!ObjectProxyWithTarget methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/30/2011 16:08'!
handler
	^ handler! !

!ObjectProxyWithTarget methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/30/2011 16:08'!
handler: anObject
	handler := anObject! !

!ObjectProxyWithTarget methodsFor: 'class membership' stamp: 'MarianoMartinezPeck 3/30/2011 16:33'!
mareaHandler
	^ handler! !

!ObjectProxyWithTarget methodsFor: 'testing' stamp: 'MarianoMartinezPeck 3/30/2011 16:05'!
mareaIsProxyInstalled
	^ target isNil not! !

!ObjectProxyWithTarget methodsFor: 'updating' stamp: 'MarianoMartinezPeck 3/30/2011 16:05'!
mareaMarkAsUninstalled
	target := nil! !

!ObjectProxyWithTarget methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/30/2011 15:50'!
target
	^ target! !

!ObjectProxyWithTarget methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 3/30/2011 15:50'!
target: anObject
	target := anObject! !

!ObjectProxyTrap classSide methodsFor: 'initialize' stamp: 'MarianoMartinezPeck 3/30/2011 15:54'!
initialize
	self nilMethodDict ! !

!ObjectProxyTrap classSide methodsFor: 'method dictionary' stamp: 'MarianoMartinezPeck 3/13/2011 19:24'!
nilMethodDict
	methodDict := nil. ! !

Error subclass: #CannotProxifyCompactClasses
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Errors'!

Error subclass: #GlobalObjectsCannotBeRoots
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Errors'!

Error subclass: #ProxyIsNotInstalled
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Errors'!

Error subclass: #SelectorNotFoundError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies-Errors'!
FilenameCompiledMethodProxySwapper initialize!
NumberAsFilenameCompiledMethodProxySwapper initialize!
NumberAsFilenameObjectProxySwapper initialize!
ObjectProxyTrap initialize!
